import sys
import pygame
from pygame.locals import *

import gamesprite
import roles
import world
import vector2

class Game:
    def __init__(self, width=1280,height=1024,fs=0,debug=0):
        pygame.init()

        #self.config()

        self.width = width
        self.height = height

        if fs == 1: self.screen = pygame.display.set_mode((self.width,self.height),FULLSCREEN)
        else: self.screen = pygame.display.set_mode((self.width,self.height))
        self.scrRect = self.screen.get_rect( )

        self.clock = pygame.time.Clock()

        self.setDebugMode(debug)
        
        self.spriteList = []

    def config(self):
        infile=open('abo.cfg','r')
        for line in infile.readlines():
            print line
        infile.close()

    def setDebugMode(self,debug):
        self.debug = debug
        gamesprite.GameSprite.debug = debug
        world.World.debug = debug
        roles.Role.debug = debug
        
    def main(self):
        
        self.world=world.World(self.scrRect.width,self.scrRect.height)
        self.world.display(self.screen)

        player=self.world.player

        movement = vector2.Vector2()

        self.world.dt = 0.016
        self.clock.tick(60)            

        while 1:
            player.speed.x=0.
            # Boucle de detection des touches appuyees
            for event in pygame.event.get():
                if event.type == pygame.QUIT: 
                    sys.exit()
                elif event.type == KEYUP:
                    if (event.key == K_DOWN):
                        if player.grounded:
                            player.jump()
                        player.fall()
                    elif (event.key == K_ESCAPE):
                        sys.exit()
            if pygame.key.get_pressed()[K_LEFT]:
                if player.grounded: player.speed.x = -150.
                else: player.speed.x = -120.
            if pygame.key.get_pressed()[K_RIGHT]:
                if player.grounded: player.speed.x = 150.
                else: player.speed.x = 120.
            if pygame.key.get_pressed()[K_DOWN]:
                if player.grounded: player.charge()
                else: player.slowFall()

            if player.jumping:
                if player.slowed:
                    if player.speed.y>0: player.speed.y=100*(1-player.jumpTime)
                    elif player.speed.y <= -50: player.speed.y=-50.
                    else: player.speed.y=player.thrust*(1-player.jumpTime)
                else: player.speed.y=player.thrust*(1-player.jumpTime)
            elif not(player.grounded) and player.slowed: player.speed.y = player.speed.y*.7
            player.speed.y -= self.world.gravity

            if self.debug: print player.speed
            movement = player.speed * self.world.dt
            movement.y = -movement.y # le 0 est en haut de l'ecran avec pygame

            movement.x=movement.x*self.world.factorH
            movement.y=movement.y*self.world.factorV
            
            # calcul des collisions
            movement = self.world.detectCollision(movement,player)
                
            player.screenPos += movement
            if self.debug: print player.worldPos

            # On met a jour l'affichage
            self.world.refresh(self.screen)
            pygame.display.flip()

            self.world.dt = float(self.clock.get_time()) / 1000
            if (player.jumping and not((player.slowed) and (player.speed.y <= -50))):
                player.jumpTime += self.world.dt
            self.clock.tick(60)
            
if __name__ == "__main__":
    #game=Game(1280,1024,1)
    #game=Game(1024,768,0)
    game=Game(800,600,0)
    game.main()
