import pygame
import yumenokaze
import vector2
import ABOMaps
import abo
import ennemies
import bonus

black = 0, 0, 0
white = 255, 255, 255
magenta = 255, 0, 255

class World:
    def __init__(self, width=800,height=600,debug=0):
        self.width = width
        self.height = height
        if self.width == 1280 and self.height == 1024:
            self.res=1
            self.offset=32
            self.factorH=1
            self.factorV=1
            self.renderWidth=40
            self.renderHeight=40
        if self.width == 1024 and self.height == 768:
            self.res=2
            self.offset=0
            self.factorH=0.8
            self.factorV=0.8
            self.renderWidth=32
            self.renderHeight=32
        if self.width == 800 and self.height == 600:
            self.res=3
            self.offset=0
            self.factorH=0.625
            self.factorV=0.625
            self.renderWidth=25
            self.renderHeight=25
        debug=0

        self.spriteList = []
        self.create()

    def create(self):
        self.map=ABOMaps.ForestForestLevel1Map()
        self.map.tileset.load(self.renderWidth,self.renderHeight)
        self.gravity=9.8

        self.background=pygame.Surface((self.width,self.height-self.offset*2))
        self.background.fill(white)

        self.mapPos=vector2.Vector2(0,self.map.mapHeight-24) #On affiche 32x24 cases en partant du bas
        self.mapDraw=vector2.Vector2(0,(self.map.mapHeight-24)*self.renderHeight)

        # affichage de la map
        for j in range(24):
            for i in range(32):
                if self.debug: print int(self.mapPos.x)+i,int(self.mapPos.y)+j,i*self.renderWidth,j*self.renderHeight,self.map.mapData[int(self.mapPos.y)+j][int(self.mapPos.x)+i]
                self.background.blit(self.map.tileset.tiles[self.map.mapData[int(self.mapPos.y)+j][int(self.mapPos.x)+i]],(i*self.renderWidth,j*self.renderHeight))

        # chargement des elements de jeux
        self.ennemies = pygame.sprite.Group()
        self.bonus= pygame.sprite.Group()
        
        for j in range(len(self.map.mapCode)):
            for i in range(len(self.map.mapCode[j])):
                if self.map.mapCode[j][i] == 1:
                    self.player=abo.ABO(i*40,j*40,self)
                    if self.debug: print "player",self.player.screenPos,self.player.worldPos,self.mapDraw
                    self.spriteList.append(self.player)
                if self.map.mapCode[j][i] == 16:
                    ennemy=ennemies.ForestWalker1(i*40,j*40,self)
                    self.spriteList.append(ennemy)
                    self.ennemies.add(ennemy)
                if self.map.mapCode[j][i] == 4:
                    bon=bonus.BGold(i*40,j*40,self)
                    self.spriteList.append(bon)
                    self.bonus.add(bon)
                if self.map.mapCode[j][i] == 3:
                    bon=bonus.BSilver(i*40,j*40,self)
                    self.spriteList.append(bon)
                    self.bonus.add(bon)
                if self.map.mapCode[j][i] == 2:
                    bon=bonus.BIron(i*40,j*40,self)
                    self.spriteList.append(bon)
                    self.bonus.add(bon)

        #self.gameSprites = pygame.sprite.Group()
        #for s in self.spriteList: self.gameSprites.add(s)

    def display(self,screen):
        screen.blit(self.background,(0,self.offset))

    def refresh(self,screen):
        # On remet a zero les zones contenant des sprites
        #for s in spriteList:
        #    screen.blit(self.background,s.rect,s.rect)

        # Scrolling
        self.background.fill((255,255,255))

        mapx = int(self.mapDraw.x) / self.renderWidth
        mapy = int(self.mapDraw.y) / self.renderHeight
        mapOffx = int(self.mapDraw.x) % self.renderWidth
        mapOffy = int(self.mapDraw.y) % self.renderHeight
        if self.debug: print mapx,mapy,mapOffx,mapOffy
        for j in range(24):
            for i in range(33):
                if self.debug: print int(self.mapPos.x)+i,int(self.mapPos.y)+j,i*self.renderWidth,j*self.renderHeight,self.map.mapData[int(self.mapPos.y)+j][int(self.mapPos.x)+i]
                self.background.blit(self.map.tileset.tiles[self.map.mapData[mapy+j][mapx+i]],(i*self.renderWidth-mapOffx,j*self.renderHeight-mapOffy))
        screen.blit(self.background,(0,self.offset))
        
        # On deplace les sprites
        for s in self.spriteList:
            s.act()
            s.move()

        # On affiche les sprites
        for s in self.spriteList: screen.blit(s.displayFrame(),s.rect)

    def move(self,spr):
        spr.worldPos=spr.screenPos+self.mapDraw
        if spr.debug: print spr.screenPos,spr.worldPos,self.mapDraw
        spr.rect.left=int(spr.screenPos.x)
        spr.rect.top=int(spr.screenPos.y+self.offset)
        if self.debug: print "move",spr.rect.left,spr.rect.top,spr.screenPos,self.factorH,self.factorV,self.offset

    def detectCollision(self,movement,player):
        tmpMovement=movement.copy()
        if self.debug: print "start",movement,tmpMovement,player.screenPos,self.mapDraw        
        if movement.y > 0: # mouvement vers le bas
            boundPosy=player.worldPos.y+self.renderHeight+movement.y
            boundPosx=player.worldPos.x
            boundPosx2=player.worldPos.x+self.renderWidth-1
            boundMapPosy=int(boundPosy)/self.renderHeight
            boundMapPosx=int(boundPosx)/self.renderWidth
            boundMapPosx2=int(boundPosx2)/self.renderWidth
            player.grounded=False # mouvement vers le bas donc on est en train de tombe sauf si on touche un sol
            if self.debug: print player.worldPos,movement.y,boundPosx,boundPosy,boundMapPosx,boundMapPosy
            if self.map.boundMapData[boundMapPosy][boundMapPosx]&1 or self.map.boundMapData[boundMapPosy][boundMapPosx2]&1:
                tmpMovement.y = movement.y-int(boundPosy)%self.renderHeight
                player.speed.y = 0.
                if player.jumping: player.thrust = 0.
                player.jumping=False
                player.jumpTime=0
                player.grounded=True
        if movement.x > 0: # mouvement vers la droitee
            boundPosx=player.worldPos.x+self.renderWidth+movement.x
            boundPosy=player.worldPos.y
            boundPosy2=player.worldPos.y+self.renderHeight-1
            boundMapPosx=int(boundPosx)/self.renderWidth
            boundMapPosy=int(boundPosy)/self.renderHeight
            boundMapPosy2=int(boundPosy2)/self.renderHeight
            if self.debug: print "bound",player.worldPos,movement.x,boundPosx,boundPosy,boundMapPosx,boundMapPosy
            if self.map.boundMapData[boundMapPosy][boundMapPosx]&2 or self.map.boundMapData[boundMapPosy2][boundMapPosx]&2:
                tmpMovement.x = movement.x-int(boundPosx)%self.renderWidth
                player.speed.x = 0.
        if movement.x < 0: # mouvement vers la gauche
            boundPosx=player.worldPos.x+movement.x
            boundPosy=player.worldPos.y
            boundPosy2=player.worldPos.y+self.renderHeight-1
            boundMapPosx=int(boundPosx)/self.renderWidth
            boundMapPosy=int(boundPosy)/self.renderHeight
            boundMapPosy2=int(boundPosy2)/self.renderHeight
            if self.debug: print "bound",player.worldPos,movement.x,boundPosx,boundPosy,boundMapPosx,boundMapPosy
            if self.map.boundMapData[boundMapPosy][boundMapPosx]&8 or self.map.boundMapData[boundMapPosy2][boundMapPosx]&8:
                tmpMovement.x = movement.x+self.renderWidth-int(boundPosx)%self.renderWidth
                player.speed.x = 0.
        if (player.screenPos.x + tmpMovement.x) > (self.width-300*self.factorH) and self.mapDraw.x < (self.map.mapWidth-33)*self.renderWidth:
            self.mapDraw.x=self.mapDraw.x+((player.screenPos.x + tmpMovement.x)-(self.width-300*self.factorH))
            tmpMovement.x=((self.width-300*self.factorH)-player.screenPos.x)
        if (player.screenPos.x + tmpMovement.x) < (300*self.factorH) and self.mapDraw.x > 0:
            self.mapDraw.x=self.mapDraw.x+((player.screenPos.x + tmpMovement.x)-(300*self.factorH))
            tmpMovement.x=((300*self.factorH)-player.screenPos.x)
        if (player.screenPos.x + tmpMovement.x) < 0:
            player.screenPos.x=0.
            tmpMovement.x=0.
        if (player.screenPos.x + self.renderWidth + tmpMovement.x) > self.width:
            player.screenPos.x=float(self.width- self.renderWidth)
            tmpMovement.x=0.
        if self.debug: print "total",movement,tmpMovement,player.screenPos,self.mapDraw
        lstColEnnemy=pygame.sprite.spritecollide(player, self.ennemies, False)
        lstColBonus=pygame.sprite.spritecollide(player, self.bonus, False)

        if lstColEnnemy:
            for e in lstColEnnemy:
                player.hit()
        
        if lstColBonus:
            for b in lstColBonus:
                player.collect(b)
                yumenokaze.deleteFromList(b,self.spriteList)
                self.bonus.remove(b)

        return tmpMovement

            

